"""Entrypoint of the CLI"""

import click

from ollama_manager.commands.delete import delete_model
from ollama_manager.commands.pull import pull_model
from ollama_manager.commands.run import run_model


@click.group()
def cli():
    pass


cli.add_command(pull_model)
cli.add_command(delete_model)
cli.add_command(run_model)



import os
import subprocess
import sys
from pathlib import Path

import click

from ollama_manager.utils import handle_interaction, list_models


def streamlit_check():
    try:
        import streamlit  # noqa F401
    except ImportError:
        print(
            """
⚠️ Running models with Streamlit UI is an optional feature.

To use it install the optional dependency:

>> pip install ollama-manager[ui]
"""
        )
        sys.exit(1)


@click.option(
    "--ui",
    "-ui",
    help="Run ollama models in a Streamlit UI",
    type=bool,
    default=False,
    is_flag=True,
)
@click.command(name="run")
def run_model(ui: bool):
    """
    Run the selected Ollama model.
    By default, uses Ollama terminal UI.

    To run model in a streamlit UI:

    >> pip install ollama-manager[ui]

    ⚠️ Only text models are supported for now.
    """
    models = list_models()
    if models:
        selection = handle_interaction(
            models, title="🚀 Select model to run:\n", multi_select=False
        )
    else:
        print("❌ No models selected for running with Streamlit UI")
        sys.exit(0)

    if selection:
        normalized_selection = selection[0].split()[0]
        if not ui:
            command = ["ollama", "run", normalized_selection]
        else:
            script_path = (
                Path(os.path.abspath(__file__)).parent.parent / "ui" / "text_chat.py"
            )

            command = ["streamlit", "run", str(script_path), "--", normalized_selection]

        try:
            process = subprocess.Popen(command)
            process.wait()
        except Exception as e:
            print(f"Error running app: {e}")



import sys

import ollama
import streamlit as st

from ollama_manager.utils import list_models

st.set_page_config(page_title="Chat: Ollama Manager")


def session_init():
    if "messages" not in st.session_state:
        st.session_state["messages"] = [
            {
                "role": "assistant",
                "content": "Hello, how can I help you?",
            }
        ]

    if "selected_model" not in st.session_state:
        st.session_state["selected_model"] = ""


def sidebar():
    with st.sidebar:
        models = list_models(only_names=True)
        selected_model = st.selectbox(
            ":material/precision_manufacturing: **:blue[Select Model]**",
            options=list_models(only_names=True),
            index=models.index(st.session_state["selected_model"] or sys.argv[1]),
        )

        if selected_model:
            st.session_state["selected_model"] = selected_model

        st.divider()
        st.slider(label="top_p", value=1.0, min_value=0.0, max_value=1.0, key="top_p")
        st.slider(
            label="top_k", value=60.0, min_value=0.0, max_value=100.0, key="top_k"
        )
        st.slider(
            label="temperature",
            value=0.7,
            min_value=0.0,
            max_value=1.0,
            key="temperature",
        )
        st.slider(
            label="Context Length",
            min_value=1000,
            max_value=9999,
            key="context_length",
            value=4000,
        )
        st.divider()
        st.caption("[:zap: Ollama Manager](https://github.com/yankeexe/ollama-manager)")
        st.caption(
            "[:bug: Report Issues](https://github.com/yankeexe/ollama-manager/issues)"
        )


def call_llm():
    messages = st.session_state["messages"]
    stream = ollama.chat(
        model=st.session_state["selected_model"] or sys.argv[1],
        stream=True,
        messages=messages,
        options={
            "temperature": st.session_state["temperature"],
            "top_k": st.session_state["top_k"],
            "top_p": st.session_state["top_p"],
            "num_ctx": st.session_state["context_length"],
        },
    )

    for chunk in stream:
        if chunk["done"] is False:
            yield chunk["message"]["content"]
        else:
            break


def run():
    session_init()
    sidebar()
    st.header("🦙 Ollama Manager: Chat Application")

    for message in st.session_state["messages"]:
        match message["role"]:
            case "assistant":
                st.chat_message("assistant").write(message["content"])
            case "user":
                st.chat_message("human").write(message["content"])

    chat_input = st.chat_input(
        placeholder="Write your message...",
    )

    if chat_input:
        st.session_state["messages"] += [{"role": "user", "content": chat_input}]
        st.chat_message("human").write(chat_input)
        response = call_llm()
        with st.chat_message("assistant"):
            ai_msg = st.write_stream(response)

        st.session_state["messages"] += [{"role": "assistant", "content": ai_msg}]


if __name__ == "__main__":
    run()



import sys

import click
import ollama

from ollama_manager.utils import handle_interaction, list_models


@click.command(name="rm")
@click.option(
    "--multi",
    "-m",
    help="Select multiple models at once",
    type=bool,
    default=False,
    is_flag=True,
)
def delete_model(multi: bool):
    """
    Deletes the selected model/s
    """
    models = list_models()
    if models:
        selections = handle_interaction(
            models, multi_select=multi, title="🗑️ Select model/s to delete:\n"
        )
    else:
        print("❌ No models selected for deletion")
        sys.exit(0)

    if selections:
        for selection in selections:
            normalized_selection = selection.split()[0]
            ollama.delete(normalized_selection)
            print(f"🗑️ Deleted model: {normalized_selection}")



import sys

import click
import ollama
import requests
from bs4 import BeautifulSoup

from ollama_manager.utils import get_session, handle_interaction, make_request


def list_remote_model_tags(model_name: str, session: requests.Session):
    response = make_request(
        session=session,
        url=f"https://ollama.com/library/{model_name}/tags",
    )
    soup = BeautifulSoup(response.text, "html.parser")

    # Find the span with the specific attribute
    # @NOTE: This might change with website updates.
    elements = soup.find_all(
        "div", class_="break-all font-medium text-gray-900 group-hover:underline"
    )

    if not elements:
        return None

    results = []
    for element in elements:
        # Get the title
        title = element.text.strip()

        # Find the next sibling div containing metadata
        # @NOTE: This might change with website updates.
        metadata_div = element.find_next(
            "div", class_="flex items-baseline space-x-1 text-[13px] text-neutral-500"
        )

        if metadata_div:
            # Extract metadata values
            metadata_text = metadata_div.text.strip()
            # Parse the metadata string
            # Example: "2887c3d03e74 • 2.5GB • 8 weeks ago"
            parts = metadata_text.split("•")
            hash_id = parts[0].strip()
            size = parts[1].strip()
            time_ago = parts[2].strip()

            results.append(
                {"title": title, "hash": hash_id, "size": size, "updated": time_ago}
            )

    return results


def list_remote_models(session: requests.Session) -> list[str] | None:
    response = make_request(session=session, url="https://ollama.com/search")

    soup = BeautifulSoup(response.text, "html.parser")
    # Find the span with the specific attribute
    # @NOTE: This might change with website updates.
    elements = soup.find_all("span", attrs={"x-test-search-response-title": True})
    if not elements:
        return None

    return [element.text.strip() for element in elements]


@click.command(name="pull")
def pull_model():
    """
    Pull models from Ollama library:

    https://ollama.dev/search
    """
    session = get_session()
    models = list_remote_models(session)
    if not models:
        print("❌ No models selected for download")
        sys.exit(0)

    model_selection = handle_interaction(
        models, title="📦 Select remote Ollama model\s:\n", multi_select=False
    )
    if model_selection:
        model_tags = list_remote_model_tags(
            model_name=model_selection[0], session=session
        )
        if not model_tags:
            print(f"❌ Failed fetching tags for: {model_selection}. Please try again.")
            sys.exit(1)

        max_length = max(len(f"{model_selection}:{tag['title']}") for tag in model_tags)

        model_name_with_tags = [
            f"{model_selection[0]}:{tag['title']:<{max_length + 5}}{tag['size']:<{max_length + 5}}{tag['updated']}"
            for tag in model_tags
        ]
        selected_model_with_tag = handle_interaction(
            model_name_with_tags, title="🔖 Select tag to download:\n"
        )
        if not selected_model_with_tag:
            print("No tag selected for the model")
            sys.exit(1)

        final_model = selected_model_with_tag[0].split()[0]
        print(f">>> Pulling model: {final_model}")
        response = ollama.pull(final_model, stream=True)
        for data in response:
            print(data)



import sys

import ollama
import requests
from simple_term_menu import TerminalMenu


def get_session() -> requests.Session:
    session = requests.Session()
    session.headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
    }

    return session


def list_models(only_names: bool = False) -> list[str] | None:
    model_names = []

    raw_models = ollama.list()
    if not raw_models:
        return None

    all_raw_models = raw_models["models"]

    max_length = max(len(model["name"]) for model in all_raw_models)

    for model in all_raw_models:
        if only_names:
            model_names.append(model["name"])
            continue

        model_names.append(
            f"{model['name']:<{max_length + 5}}{convert_bytes(model['size'])}"
        )

    return model_names


def make_request(session: requests.Session, url: str, timeout: int = 5):
    try:
        response = session.get(url, timeout=timeout)
        response.raise_for_status()
    except requests.exceptions.RequestException as e:
        print(f"❌ Failed to make request: {str(e)}")
        sys.exit(1)

    return response


def convert_bytes(bytes_value):
    """
    Convert bytes to human readable format (MB or GB).

    Args:
        bytes_value (int): Size in bytes to convert

    Returns:
        str: Formatted string with size in MB or GB (e.g. "1.50 MB" or "2.30 GB")
            Returns MB if size is less than 1 GB, otherwise returns GB
    """
    KB = 1024
    MB = KB * 1024
    GB = MB * 1024

    # Convert to MB first
    mb_value = bytes_value / MB

    if mb_value < 1024:  # Less than 1 GB
        return f"{mb_value:.2f} MB"
    else:  # 1 GB or more
        gb_value = bytes_value / GB
        return f"{gb_value:.2f} GB"


def handle_interaction(
    data: list, multi_select=False, title: str | None = None
) -> list[str]:
    """
    Display interactive menu on the terminal.
    """
    selections = []
    try:
        terminal_menu = TerminalMenu(
            data,
            multi_select=multi_select,
            show_multi_select_hint=multi_select,
            search_key=None,
            show_search_hint=True,
            title=title,
        )
        menu_entry_index: tuple | None = terminal_menu.show()

        # Check for None value when user presses `esc` or `ctrl + c`
        if menu_entry_index is None:
            raise KeyboardInterrupt

        if multi_select:
            # if more than one timezone is selected.
            for index in menu_entry_index:
                selections.append(data[index])
        else:
            selections.append(data[menu_entry_index])

    except KeyboardInterrupt:
        sys.exit()

    return selections
